/* slist.c
 *
 *  Copyright (C) 2006,2007 Tuomo Makinen (tjam@users.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "slist.h"

static void slist_allocate_node (struct node **new_node);

struct list_s *
slist_create ()
{
  struct list_s *new;

  new = (struct list_s *) malloc (sizeof (struct list_s));
  if (new == NULL)
    return NULL;
  else
    {
      new->head = NULL;
      new->tail = NULL;
    }
  return new;
}

struct node *
slist_push (struct list_s *list)
{
  struct node *new_node;

  slist_allocate_node (&new_node);

  if (new_node == NULL)
    return NULL;
    
  new_node->next = list->head;
  if (list->head == NULL)
    list->tail = new_node;
  list->head = new_node;
  
  return new_node;
}

struct node *
slist_append (struct list_s *list)
{
  struct node *new_node;

  slist_allocate_node (&new_node);
  if (new_node == NULL)
    return NULL;

  new_node->next = NULL;

  if (list->head == NULL)
    {
      list->head = new_node;
      list->tail = new_node;
    }
  else
    {
      list->tail->next = new_node;
      list->tail = new_node;
    }
  return new_node;
}

int
slist_length (struct list_s *list)
{
  struct node *current;
  int count;

  count = 0;
  current = list->head;

  while (current != NULL)
    {
      count++;
      current = current->next;
    }
  return count;
}

void
slist_clear (struct list_s *list)
{
  struct node *current;
  struct node *next;

  current = list->head;

  while (current != NULL)
    {
      next = current->next;
      free (current->payload);
      free (current);

      current = next;
    }
  list->head = NULL;
  list->tail = NULL;
}

void
slist_delete (struct list_s *list)
{
  slist_clear (list);
  free (list);
}

static void
slist_allocate_node (struct node **new_node)
{
  *new_node = (struct node *) malloc (sizeof (struct node));
}

